/*
 * taskview-io.c
 *
 * TaskviewIO
 *
 * Copyright © 2010 Salomon Sickert
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 3 of the licence or (at
 * your option) any later version.
 *
 * See the included COPYING.LESSER file for more information.
 *
 * Authors:
 *    Salomon Sickert <sickert@in.tum.de>
 */

#include "taskview.h"
#include "taskview-io.h"
#include "taskview-manager.h"

#define TASKVIEW_IO_GET_PRIVATE(obj)	(G_TYPE_INSTANCE_GET_PRIVATE ((obj), TASKVIEW_TYPE_IO, TaskviewIOPrivate))

G_DEFINE_TYPE (TaskviewIO, taskview_io, TASKVIEW_TYPE_GENERIC);

#define TV_IO_UPDATE_INTERVAL 1000
#define TV_IO_MIN_TIME_ELAPSED 15

static gboolean   update_eta (gpointer *user_data);
static void       update_progress (TaskviewIO *self);
static GVariant * g_variant_new_empty_as (void);

struct _TaskviewIOPrivate
{
  TaskviewManager *manager;
  gint type;

  guint current_item;
  GVariant *destination_uri;
  GVariant *source_uri;

  guint64 throughput_size;
  guint64 processed_size;
  guint64 total_size;

  guint64 throughput_items;
  guint64 processed_items;
  guint64 total_items;

  GTimer *elapsed_time;
  guint auto_update;
};

/* ----------------------------------------------------------------------------------------------------
 * GDBus Interface Specification
 * ---------------------------------------------------------------------------------------------------- */

static GDBusNodeInfo *introspection_data = NULL;
static const gchar introspection_xml[] =
  "<node>"
  "  <interface name=\"org.gnome.TaskView.IO\">"
  "    <property type=\"i\" name=\"Type\"            access=\"read\" />"
  "    <property type=\"u\" name=\"CurrentItem\"     access=\"read\" />"
  "    <property type=\"as\" name=\"DestinationURI\" access=\"read\" />"
  "    <property type=\"as\" name=\"SourceURI\"      access=\"read\" />"
  "    <property type=\"t\" name=\"ThroughputSize\"  access=\"read\" />"
  "    <property type=\"t\" name=\"ProcessedSize\"   access=\"read\" />"
  "    <property type=\"t\" name=\"TotalSize\"       access=\"read\" />"
  "    <property type=\"t\" name=\"ThroughputItems\" access=\"read\" />"
  "    <property type=\"t\" name=\"ProcessedItems\"  access=\"read\" />"
  "    <property type=\"t\" name=\"TotalItems\"      access=\"read\" />"
  "  </interface>"
  "</node>";

/* ----------------------------------------------------------------------------------------------------
 * GObject Properties
 * ---------------------------------------------------------------------------------------------------- */

enum
{
  PROP_0,

  PROP_TYPE,
  PROP_CURRENT_ITEM,
  PROP_DESTINATION_URI,
  PROP_SOURCE_URI,

  PROP_THROUGHPUT_SIZE,
  PROP_PROCESSED_SIZE,
  PROP_TOTAL_SIZE,

  PROP_THROUGHPUT_ITEMS,
  PROP_PROCESSED_ITEMS,
  PROP_TOTAL_ITEMS,

  PROP_AUTO_UPDATE
};

static void
taskview_io_set_property (GObject      *object,
                          guint         property_id,
                          const GValue *value,
                          GParamSpec   *pspec)
{
  TaskviewIO *self = TASKVIEW_IO (object);

  gpointer boxed;
  gchar *object_path;
  GVariantBuilder *invalidated_properties;
  GVariantBuilder *changed_properties;

  invalidated_properties = g_variant_builder_new (G_VARIANT_TYPE("as"));
  changed_properties = g_variant_builder_new (G_VARIANT_TYPE_ARRAY);

  switch (property_id)
    {
    case PROP_TYPE:
      if (self->priv->type == g_value_get_int (value)) return;
      self->priv->type = g_value_get_int (value);
      g_variant_builder_add (changed_properties, "{sv}", "Type", g_variant_new_int32 (self->priv->type));
      break;

    case PROP_CURRENT_ITEM:
      if (self->priv->current_item == g_value_get_uint (value)) return;
      self->priv->current_item = g_value_get_uint (value);
      g_variant_builder_add (changed_properties, "{sv}", "Type", g_variant_new_uint32 (self->priv->current_item));
      break;

    case PROP_DESTINATION_URI:
      g_variant_unref (self->priv->destination_uri);
      boxed = g_value_get_boxed (value);
      if (boxed != NULL)
        self->priv->destination_uri = g_variant_ref_sink (g_variant_new_strv (boxed, -1));
      else
        self->priv->destination_uri = g_variant_new_empty_as ();
      g_variant_builder_add (changed_properties, "{sv}", "DestinationURI", g_variant_ref (self->priv->destination_uri));
      break;

    case PROP_SOURCE_URI:
      g_variant_unref (self->priv->source_uri);
      boxed = g_value_get_boxed (value);
      if (boxed != NULL)
        self->priv->source_uri = g_variant_ref_sink (g_variant_new_strv (boxed, -1));
      else
        self->priv->source_uri = g_variant_new_empty_as ();
      g_variant_builder_add (changed_properties, "{sv}", "SourceURI", self->priv->source_uri);
      break;

    case PROP_THROUGHPUT_SIZE:
      if (self->priv->throughput_size == g_value_get_uint64 (value)) return;
      self->priv->throughput_size = g_value_get_uint64 (value);
      g_variant_builder_add (changed_properties, "{sv}", "ThroughputSize", g_variant_new_uint64 (self->priv->throughput_size));
      break;

    case PROP_PROCESSED_SIZE:
      if (self->priv->processed_size == g_value_get_uint64 (value)) return;
      self->priv->processed_size = g_value_get_uint64 (value);
      g_variant_builder_add (changed_properties, "{sv}", "ProcessedSize", g_variant_new_uint64 (self->priv->processed_size));
      update_progress (self);
      break;

    case PROP_TOTAL_SIZE:
      if (self->priv->total_size == g_value_get_uint64 (value)) return;
      self->priv->total_size = g_value_get_uint64 (value);
      g_variant_builder_add (changed_properties, "{sv}", "TotalSize", g_variant_new_uint64 (self->priv->total_size));
      update_progress (self);
      break;

    case PROP_THROUGHPUT_ITEMS:
      if (self->priv->throughput_items == g_value_get_uint64 (value)) return;
      self->priv->throughput_items = g_value_get_uint64 (value);
      g_variant_builder_add (changed_properties, "{sv}", "ThroughputItems", g_variant_new_uint64 (self->priv->throughput_items));
      break;

    case PROP_PROCESSED_ITEMS:
      if (self->priv->processed_items == g_value_get_uint64 (value)) return;
      self->priv->processed_items = g_value_get_uint64 (value);
      g_variant_builder_add (changed_properties, "{sv}", "ProcessedItems", g_variant_new_uint64 (self->priv->processed_items));
      update_progress (self);
      break;

    case PROP_TOTAL_ITEMS:
      if (self->priv->total_items == g_value_get_uint64 (value)) return;
      self->priv->total_items = g_value_get_uint64 (value);
      g_variant_builder_add (changed_properties, "{sv}", "TotalItems", g_variant_new_uint64 (self->priv->total_items));
      update_progress (self);
      break;

    case PROP_AUTO_UPDATE:
      if (g_value_get_boolean (value) && self->priv->auto_update == 0)
        {
          self->priv->auto_update = g_timeout_add (TV_IO_UPDATE_INTERVAL, (GSourceFunc) update_eta, self);
        }
      else if (!g_value_get_boolean (value) && self->priv->auto_update != 0)
        {
          g_source_remove (self->priv->auto_update);
          self->priv->auto_update = 0;
        }
      break;

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
      break;
    }

  /* FIXME */
  if (property_id <= PROP_TOTAL_ITEMS)
    {
      g_object_get (self, "object-path", &object_path, NULL);
      taskview_manager_property_changed (self->priv->manager, object_path, TASKVIEW_IO_INTERFACE_NAME, changed_properties, invalidated_properties);
      g_free (object_path);
    }

  g_variant_builder_unref (changed_properties);
  g_variant_builder_unref (invalidated_properties);
}

static void
taskview_io_get_property (GObject    *object,
                          guint       property_id,
                          GValue     *value,
                          GParamSpec *pspec)
{
  TaskviewIO *self = TASKVIEW_IO (object);

  switch (property_id)
    {
    case PROP_TYPE:
      g_value_set_int (value, self->priv->type);
      break;

    case PROP_CURRENT_ITEM:
      g_value_set_uint (value, self->priv->current_item);
      break;

    case PROP_DESTINATION_URI:
      g_value_set_boxed_take_ownership (value, g_variant_get_strv (self->priv->destination_uri, NULL));
      break;

    case PROP_SOURCE_URI:
      g_value_set_boxed_take_ownership (value, g_variant_get_strv (self->priv->source_uri, NULL));
      break;

    case PROP_THROUGHPUT_SIZE:
      g_value_set_uint64 (value, self->priv->throughput_size);
      break;

    case PROP_PROCESSED_SIZE:
      g_value_set_uint64 (value, self->priv->processed_size);
      break;

    case PROP_TOTAL_SIZE:
      g_value_set_uint64 (value, self->priv->total_size);
      break;

    case PROP_THROUGHPUT_ITEMS:
      g_value_set_uint64 (value, self->priv->throughput_items);
      break;

    case PROP_PROCESSED_ITEMS:
      g_value_set_uint64 (value, self->priv->processed_items);
      break;

    case PROP_TOTAL_ITEMS:
      g_value_set_uint64 (value, self->priv->total_items);
      break;

    case PROP_AUTO_UPDATE:
      g_value_set_boolean (value, self->priv->auto_update != 0);
      break;

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
      break;
    }
}


/* --------------------------------------------------------------------------
 * D-Bus
 * -------------------------------------------------------------------------- */

static GVariant *
handle_get_property (GDBusConnection  *connection,
                     const gchar      *sender,
                     const gchar      *object_path,
                     const gchar      *interface_name,
                     const gchar      *property_name,
                     GError          **error,
                     gpointer          user_data)
{
  TaskviewIO *self;
  GVariant *ret;

  g_assert (TASKVIEW_IS_IO (user_data));
  self = TASKVIEW_IO (user_data);

  ret = NULL;

  if (g_strcmp0 (property_name, "Type") == 0)
    {
      ret = g_variant_new_int32 (self->priv->type);
    }
  else if (g_strcmp0 (property_name, "CurrentItem") == 0)
    {
      ret = g_variant_new_uint32 (self->priv->current_item);
    }
  else if (g_strcmp0 (property_name, "DestinationURI") == 0)
    {
      ret = g_variant_ref (self->priv->destination_uri);
    }
  else if (g_strcmp0 (property_name, "SourceURI") == 0)
    {
      ret = g_variant_ref (self->priv->source_uri);
    }
  else if (g_strcmp0 (property_name, "ThroughputSize") == 0)
    {
      ret = g_variant_new_uint64 (self->priv->throughput_size);
    }
  else if (g_strcmp0 (property_name, "ProcessedSize") == 0)
    {
      ret = g_variant_new_uint64 (self->priv->processed_size);
    }
  else if (g_strcmp0 (property_name, "TotalSize") == 0)
    {
      ret = g_variant_new_uint64 (self->priv->total_size);
    }
  else if (g_strcmp0 (property_name, "ThroughputItems") == 0)
    {
      ret = g_variant_new_uint64 (self->priv->throughput_items);
    }
  else if (g_strcmp0 (property_name, "ProcessedItems") == 0)
    {
      ret = g_variant_new_uint64 (self->priv->processed_items);
    }
  else if (g_strcmp0 (property_name, "TotalItems") == 0)
    {
      ret = g_variant_new_uint64 (self->priv->total_items);
    }
  else
    {
      g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS, "Unknown Property");
    }

  return ret;
}

const GDBusInterfaceVTable taskview_io_interface_vtable =
{
  NULL,
  handle_get_property,
  NULL
};

/* --------------------------------------------------------------------------
 * Helper
 * -------------------------------------------------------------------------- */

static gint
calculate_progress (guint64 processed, guint64 total)
{
  if (total != 0 &&
      total != G_MAXUINT64 &&
      processed != G_MAXUINT64 &&
      processed <= total)
    {
      return (processed * 100) / total;
    }
  else
    {
      return -1;
    }
}

static guint64
calculate_throughput (guint64 processed, gdouble elapsed_time)
{
  if (processed != G_MAXUINT64)
    {
      return (guint64) (processed / elapsed_time);
    }
  else
    {
      return G_MAXUINT64;
    }
}

static gint
calculate_eta (guint64 processed, guint64 total, guint64 throughput)
{
  if (total != 0 &&
      total != G_MAXUINT64 &&
      processed <= total &&
      processed != G_MAXUINT64 &&
      throughput != 0 &&
      throughput != G_MAXUINT64)
    {
      return (gint) ((total - processed) / throughput);
    }
  else
    {
      return -1;
    }
}

static void
update_progress (TaskviewIO *self)
{
  gint progress_size, progress_items;

  if (!self->priv->auto_update)
    return;

  progress_size = calculate_progress (self->priv->processed_size, self->priv->total_size);
  progress_items = calculate_progress (self->priv->processed_items, self->priv->total_items);

  g_object_set (self,
                "progress", progress_size != -1 ? progress_size : progress_items,
                NULL);
}

static gboolean
update_eta (gpointer *user_data)
{
  TaskviewIO *self = TASKVIEW_IO (user_data);
  gdouble elapsed;
  gint eta_size, eta_items;
  guint64 throughput_size, throughput_items;

  if (self == NULL || !(self->priv->auto_update))
    return FALSE;

  if (self->priv->elapsed_time == NULL)
    return TRUE;

  elapsed = g_timer_elapsed (self->priv->elapsed_time, NULL);

  if (elapsed < TV_IO_MIN_TIME_ELAPSED)
    return TRUE;

  throughput_size = calculate_throughput (self->priv->processed_size, elapsed);
  throughput_items = calculate_throughput (self->priv->processed_items, elapsed);

  eta_size = calculate_eta (self->priv->processed_size, self->priv->total_size, throughput_size);
  eta_items = calculate_eta (self->priv->processed_items, self->priv->total_items, throughput_items);

  g_object_set (self,
                "throughput-size", throughput_size,
                "throughput-items", throughput_items,
                "remaining-time", throughput_size != G_MAXUINT64 ? eta_size : eta_items,
                NULL);

  return TRUE;
}

static void
timer_update (GObject *caller, GParamSpec *arg1, gpointer user_data)
{
  TaskviewIO *self = TASKVIEW_IO (caller);
  gint state;

  g_object_get (self, "state", &state, NULL);

  if (state != TASKVIEW_RUNNING && self->priv->elapsed_time != NULL)
    {
      g_timer_destroy (self->priv->elapsed_time);
      self->priv->elapsed_time = NULL;
    }
  else if (state == TASKVIEW_RUNNING && self->priv->elapsed_time == NULL)
    {
      self->priv->elapsed_time = g_timer_new ();
    }
}

static GVariant *
g_variant_new_empty_as (void)
{
  const char *empty[] = {NULL};
  return g_variant_ref_sink (g_variant_new_strv (empty, -1));
}

/* --------------------------------------------------------------------------
 * GObject
 * -------------------------------------------------------------------------- */

static void
taskview_io_dispose (GObject *gobject)
{
  TaskviewIO *self = TASKVIEW_IO (gobject);

  if (self->priv->manager != NULL)
    {
      g_object_unref (self->priv->manager);
      self->priv->manager = NULL;
    }

  if (self->priv->auto_update != 0)
    {
      g_source_remove (self->priv->auto_update);
      self->priv->auto_update = 0;
    }

  if (self->priv->destination_uri != NULL)
    {
      g_variant_unref (self->priv->destination_uri);
      self->priv->destination_uri = NULL;
    }

  if (self->priv->source_uri != NULL)
    {
      g_variant_unref (self->priv->source_uri);
      self->priv->source_uri = NULL;
    }

  if (self->priv->elapsed_time != NULL)
    {
      g_timer_destroy (self->priv->elapsed_time);
      self->priv->elapsed_time = NULL;
    }

  G_OBJECT_CLASS (taskview_io_parent_class)->dispose (gobject);
}

static void
taskview_io_finalize (GObject *gobject)
{
  TaskviewIO *self = TASKVIEW_IO (gobject);

  G_OBJECT_CLASS (taskview_io_parent_class)->finalize (gobject);
}

static void
taskview_io_class_init (TaskviewIOClass *klass)
{
  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
  GParamSpec *pspec;

  g_type_class_add_private (gobject_class, sizeof (TaskviewIOPrivate));

  /* Build D-Bus Introspection */
  introspection_data = g_dbus_node_info_new_for_xml (introspection_xml, NULL);
  g_assert (introspection_data != NULL);
  klass->interface = g_dbus_node_info_lookup_interface (introspection_data, TASKVIEW_IO_INTERFACE_NAME);
  g_assert (klass->interface != NULL);
  klass->vtable = &taskview_io_interface_vtable;

  gobject_class->dispose = taskview_io_dispose;
  gobject_class->finalize = taskview_io_finalize;
  gobject_class->set_property = taskview_io_set_property;
  gobject_class->get_property = taskview_io_get_property;

  /* Type */
  pspec = g_param_spec_int  ("type",
                             "Type",
                             "The type of the IO transfer",
                             0,
                             TASKVIEW_IO_UNKNOWN,
                             TASKVIEW_IO_UNKNOWN,
                             G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_TYPE, pspec);

  /* CurrentItem */
  pspec = g_param_spec_uint ("current-item",
                             "CurrentItem",
                             "The index of the current processed item, contained in DestinationURI",
                             0,
                             G_MAXUINT32,
                             G_MAXUINT32,
                             G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_CURRENT_ITEM, pspec);

  /* DestinationURI */
  pspec = g_param_spec_boxed ("destination-uri",
                              "DestinationURI",
                              "All destination uris",
                              G_TYPE_STRV,
                              G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_DESTINATION_URI, pspec);

  /* SourceURI */
  pspec = g_param_spec_boxed ("source-uri",
                              "SourceURI",
                              "All source uris",
                              G_TYPE_STRV,
                              G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_SOURCE_URI, pspec);

  /* ThroughputSize */
  pspec = g_param_spec_uint64  ("throughput-size",
                                "ThroughputSize",
                                "The size throughput of the Task",
                                0,
                                G_MAXUINT64,
                                G_MAXUINT64,
                                G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_THROUGHPUT_SIZE, pspec);

  /* ProcessedSize */
  pspec = g_param_spec_uint64  ("processed-size",
                                "ProcessedSize",
                                "The processed size of the Task",
                                0,
                                G_MAXUINT64,
                                G_MAXUINT64,
                                G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_PROCESSED_SIZE, pspec);

  /* TotalSize */
  pspec = g_param_spec_uint64  ("total-size",
                                "TotalSize",
                                "The total size of the Task",
                                0,
                                G_MAXUINT64,
                                G_MAXUINT64,
                                G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_TOTAL_SIZE, pspec);

  /* ThroughputItems */
  pspec = g_param_spec_uint64  ("throughput-items",
                                "ThroughputItems",
                                "The items throughput of the Task",
                                0,
                                G_MAXUINT64,
                                G_MAXUINT64,
                                G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_THROUGHPUT_ITEMS, pspec);

  /* ProcessedItems */
  pspec = g_param_spec_uint64  ("processed-items",
                                "ProcessedItems",
                                "The number processed items of the Task",
                                0,
                                G_MAXUINT64,
                                G_MAXUINT64,
                                G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_PROCESSED_ITEMS, pspec);

  /* TotalItems */
  pspec = g_param_spec_uint64  ("total-items",
                                "TotalFiles",
                                "The number of all items of the Task",
                                0,
                                G_MAXUINT64,
                                G_MAXUINT64,
                                G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_TOTAL_ITEMS, pspec);

  /* Auto Update */
  pspec = g_param_spec_boolean ("auto-update",
                                "AutoUpdate",
                                "Automatic updates of the progress, ETA, throughput props from the size information",
                                TRUE,
                                G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
  g_object_class_install_property (gobject_class, PROP_AUTO_UPDATE, pspec);
}

static void
taskview_io_init (TaskviewIO *self)
{
  self->priv = TASKVIEW_IO_GET_PRIVATE (self);
  self->priv->manager = taskview_manager_dup ();
  self->priv->elapsed_time = g_timer_new ();
  self->priv->destination_uri = g_variant_new_empty_as ();
  self->priv->source_uri = g_variant_new_empty_as ();
  g_signal_connect (self, "notify::state", G_CALLBACK (timer_update), NULL);
  timer_update (G_OBJECT (self), NULL, NULL);
}

/* ----------------------------------------------------------------------------------------------------
 * C Wrapper for g_object_set/get/new
 * ---------------------------------------------------------------------------------------------------- */

TaskviewIO *
taskview_io_new (void)
{
  return g_object_new (TASKVIEW_TYPE_IO, NULL);
}

gint
taskview_io_get_io_type (TaskviewIO *self)
{
  gint value;
  g_object_get (self, "type", &value, NULL);
  return value;
}

guint
taskview_io_get_current_item (TaskviewIO *self)
{
  guint value;
  g_object_get (self, "current-item", &value, NULL);
  return value;
}

GStrv
taskview_io_get_destination_uri (TaskviewIO *self)
{
  GStrv value;
  g_object_get (self, "destination-uri", &value, NULL);
  return value;
}

GStrv
taskview_io_get_source_uri (TaskviewIO *self)
{
  GStrv value;
  g_object_get (self, "source-uri", &value, NULL);
  return value;
}

guint64
taskview_io_get_throughput_size (TaskviewIO *self)
{
  guint64 value;
  g_object_get (self, "throughput-size", &value, NULL);
  return value;
}

guint64
taskview_io_get_total_size (TaskviewIO *self)
{
  guint64 value;
  g_object_get (self, "total-size", &value, NULL);
  return value;
}

guint64
taskview_io_get_processed_size (TaskviewIO *self)
{
  guint64 value;
  g_object_get (self, "processed-size", &value, NULL);
  return value;
}

guint64
taskview_io_get_throughput_items (TaskviewIO *self)
{
  guint64 value;
  g_object_get (self, "throughput-items", &value, NULL);
  return value;
}

guint64
taskview_io_get_total_items (TaskviewIO *self)
{
  guint64 value;
  g_object_get (self, "total-items", &value, NULL);
  return value;
}

guint64
taskview_io_get_processed_items (TaskviewIO *self)
{
  guint64 value;
  g_object_get (self, "processed-items", &value, NULL);
  return value;
}

gboolean
taskview_io_get_auto_update (TaskviewIO *self)
{
  gboolean value;
  g_object_get (self, "auto-update", &value, NULL);
  return value;
}

void
taskview_io_set_io_type (TaskviewIO *self, gint value)
{
  g_return_if_fail (TASKVIEW_IS_IO (self));
  g_object_set (self, "type", value, NULL);
}

void
taskview_io_set_current_item (TaskviewIO *self, guint value)
{
  g_return_if_fail (TASKVIEW_IS_IO (self));
  g_object_set (self, "current-item", value, NULL);
}

void
taskview_io_set_destination_uri (TaskviewIO *self, const GStrv value)
{
  g_return_if_fail (TASKVIEW_IS_IO (self));
  g_object_set (self, "destination-uri", value, NULL);
}

void
taskview_io_set_source_uri (TaskviewIO *self, const GStrv value)
{
  g_return_if_fail (TASKVIEW_IS_IO (self));
  g_object_set (self, "source-uri", value, NULL);
}

void
taskview_io_set_throughput_size (TaskviewIO *self, guint64 value)
{
  g_return_if_fail (TASKVIEW_IS_IO (self));
  g_object_set (self, "throughput-size", value, NULL);
}

void
taskview_io_set_total_size (TaskviewIO *self, guint64 value)
{
  g_return_if_fail (TASKVIEW_IS_IO (self));
  g_object_set (self, "total-size", value, NULL);
}

void
taskview_io_set_processed_size (TaskviewIO *self, guint64 value)
{
  g_return_if_fail (TASKVIEW_IS_IO (self));
  g_object_set (self, "processed-size", value, NULL);
}

void
taskview_io_set_throughput_items (TaskviewIO *self, guint64 value)
{
  g_return_if_fail (TASKVIEW_IS_IO (self));
  g_object_set (self, "throughput-items", value, NULL);
}

void
taskview_io_set_total_items (TaskviewIO *self, guint64 value)
{
  g_return_if_fail (TASKVIEW_IS_IO (self));
  g_object_set (self, "total-items", value, NULL);
}

void
taskview_io_set_processed_items (TaskviewIO *self, guint64 value)
{
  g_return_if_fail (TASKVIEW_IS_IO (self));
  g_object_set (self, "processed-items", value, NULL);
}

void
taskview_io_set_auto_update (TaskviewIO *self, gboolean value)
{
  g_return_if_fail (TASKVIEW_IS_IO (self));
  g_object_set (self, "auto-update", value, NULL);
}
